@@ -22,7 +22,7 @@
 	This file incorporates work covered by the following copyright and
 	permission notice:
 
-	Copyright (c) 2013-2019, 2021 Cong Xu
+	Copyright (c) 2013-2019, 2021, 2023 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -108,7 +108,8 @@ BulletClass *IdBulletClass(const int i)
 	{
 		return CArrayGet(&gBulletClasses.Classes, i);
 	}
-	return CArrayGet(&gBulletClasses.CustomClasses, i - gBulletClasses.Classes.size);
+	return CArrayGet(
+		&gBulletClasses.CustomClasses, i - gBulletClasses.Classes.size);
 }
 
 // Draw functions
@@ -643,8 +644,8 @@ static void OnHit(HitItemData *data, Thing *target)
 	data->HitType = GetHitType(target, data->Obj, &targetUID);
 	const TActor *source = ActorGetByUID(data->Obj->ActorUID);
 	Damage(
-		data->Obj->thing.Vel, data->Obj->bulletClass, data->Obj->flags, source,
-		target->kind, targetUID);
+		data->Obj->thing.Vel, data->Obj->bulletClass, data->Obj->weapon,
+		data->Obj->flags, source, target->kind, targetUID);
 	if (target->SoundLock <= 0)
 	{
 		target->SoundLock += SOUND_LOCK_THING;
@@ -883,6 +884,7 @@ static void LoadBullet(
 	{
 		json_t *falling = json_find_first_label(node, "Falling")->child;
 		LoadFloat(&b->Falling.GravityFactor, falling, "GravityFactor");
+		b->Falling.FallsDown = true;
 		LoadBool(&b->Falling.FallsDown, falling, "FallsDown");
 		LoadBool(&b->Falling.DestroyOnDrop, falling, "DestroyOnDrop");
 		LoadBool(&b->Falling.Bounces, falling, "Bounces");
@@ -1082,6 +1084,8 @@ void BulletAdd(const NAddBullet add)
 		obj->flags |= FLAGS_HURTALWAYS;
 	}
 
+	obj->weapon = StrWeaponClass(add.Gun);
+
 	obj->isInUse = true;
 	obj->thing.drawFunc = NULL;
 	obj->thing.drawData.MobObjId = i;
@@ -1089,6 +1093,18 @@ void BulletAdd(const NAddBullet add)
 	obj->thing.CPicFunc = BulletDraw;
 	obj->thing.ShadowSize = obj->bulletClass->ShadowSize;
 	MapTryMoveThing(&gMap, &obj->thing, pos);
+
+	// Update shots
+	if (!gCampaign.IsClient)
+	{
+		const TActor *source = ActorGetByUID(add.ActorUID);
+		const int playerUID = source != NULL ? source->PlayerUID : -1;
+		PlayerData *p = PlayerDataGetByUID(playerUID);
+		if (p)
+		{
+			WeaponUsagesUpdate(p->WeaponUsages, obj->weapon, 1, 0);
+		}
+	}
 }
 
 void BulletBounce(const NBulletBounce bb)
@@ -1125,24 +1141,30 @@ void BulletBounce(const NBulletBounce bb)
 void PlayHitSound(const BulletClass *b, const HitType t, const struct vec2 pos)
 {
 	GameEvent es = GameEventNew(GAME_EVENT_SOUND_AT);
+	const char *sound = NULL;
 	switch (t)
 	{
 	case HIT_NONE:
 		// Do nothing
 		return;
 	case HIT_WALL:
-		strcpy(es.u.SoundAt.Sound, b->Hit.Wall.Sound);
+		sound = b->Hit.Wall.Sound;
 		break;
 	case HIT_OBJECT:
-		strcpy(es.u.SoundAt.Sound, b->Hit.Object.Sound);
+		sound = b->Hit.Object.Sound;
 		break;
 	case HIT_FLESH:
-		strcpy(es.u.SoundAt.Sound, b->Hit.Flesh.Sound);
+		sound = b->Hit.Flesh.Sound;
 		break;
 	default:
 		CASSERT(false, "unknown hit type")
 		return;
 	}
+	if (!sound || strlen(sound) == 0)
+	{
+		return;
+	}
+	strcpy(es.u.SoundAt.Sound, sound);
 	es.u.SoundAt.Pos = Vec2ToNet(pos);
 	GameEventsEnqueue(&gGameEvents, es);
 }
