@@ -1,7 +1,7 @@
 /*
 	C-Dogs SDL
 	A port of the legendary (and fun) action/arcade cdogs.
-	Copyright (c) 2014-2016, 2018-2020, 2022-2023 Cong Xu
+	Copyright (c) 2014-2016, 2018-2020, 2022-2023, 2025 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -62,6 +62,8 @@ void PlayerDataAddOrUpdate(const NPlayerData pd)
 
 		p->Char.speed = 1;
 
+		p->WeaponUsages = WeaponUsagesNew();
+
 		LOG(LM_MAIN, LL_INFO, "add default player UID(%u) local(%s)", pd.UID,
 			p->IsLocal ? "true" : "false");
 	}
@@ -74,12 +76,17 @@ void PlayerDataAddOrUpdate(const NPlayerData pd)
 	{
 		p->Char.Class = StrCharacterClass("Jones");
 	}
-	CFREE(p->Char.Hair);
-	p->Char.Hair = NULL;
-	if (strlen(pd.Hair) > 0)
-	{
-		CSTRDUP(p->Char.Hair, pd.Hair);
+#define ADDHEADPART(_hp, _pdPart)                                             \
+	CFREE(p->Char.HeadParts[_hp]);                                            \
+	p->Char.HeadParts[_hp] = NULL;                                            \
+	if (strlen(_pdPart) > 0)                                                  \
+	{                                                                         \
+		CSTRDUP(p->Char.HeadParts[_hp], _pdPart);                             \
 	}
+	ADDHEADPART(HEAD_PART_HAIR, pd.Hair);
+	ADDHEADPART(HEAD_PART_FACEHAIR, pd.Facehair);
+	ADDHEADPART(HEAD_PART_HAT, pd.Hat);
+	ADDHEADPART(HEAD_PART_GLASSES, pd.Glasses);
 	p->Char.Colors = Net2CharColors(pd.Colors);
 	for (int i = 0; i < (int)pd.Weapons_count; i++)
 	{
@@ -99,13 +106,16 @@ void PlayerDataAddOrUpdate(const NPlayerData pd)
 	p->Stats = pd.Stats;
 	p->Totals = pd.Totals;
 	p->Char.maxHealth = pd.MaxHealth;
+	p->Char.excessHealth = pd.ExcessHealth;
+	p->HP = pd.HP;
 	p->lastMission = pd.LastMission;
 
 	// Ready players as well
 	p->Ready = true;
 
-	LOG(LM_MAIN, LL_INFO, "update player UID(%d) maxHealth(%d)", p->UID,
-		p->Char.maxHealth);
+	LOG(LM_MAIN, LL_INFO,
+		"update player UID(%d) maxHealth(%d) excessHealth(%d)  HP(%d)", p->UID,
+		p->Char.maxHealth, p->Char.excessHealth, p->HP);
 }
 
 static void PlayerTerminate(PlayerData *p);
@@ -140,6 +150,7 @@ void PlayerRemove(const int uid)
 static void PlayerTerminate(PlayerData *p)
 {
 	CFREE(p->Char.bot);
+	WeaponUsagesTerminate(p->WeaponUsages);
 }
 
 NPlayerData PlayerDataDefault(const int idx)
@@ -154,9 +165,21 @@ NPlayerData PlayerDataDefault(const int idx)
 	{
 		strcpy(pd.Name, t->name);
 		strcpy(pd.CharacterClass, t->CharClassName);
-		if (t->Hair != NULL)
+		if (t->HeadParts[HEAD_PART_HAIR] != NULL)
 		{
-			strcpy(pd.Hair, t->Hair);
+			strcpy(pd.Hair, t->HeadParts[HEAD_PART_HAIR]);
+		}
+		if (t->HeadParts[HEAD_PART_FACEHAIR] != NULL)
+		{
+			strcpy(pd.Facehair, t->HeadParts[HEAD_PART_FACEHAIR]);
+		}
+		if (t->HeadParts[HEAD_PART_HAT] != NULL)
+		{
+			strcpy(pd.Hat, t->HeadParts[HEAD_PART_HAT]);
+		}
+		if (t->HeadParts[HEAD_PART_GLASSES] != NULL)
+		{
+			strcpy(pd.Glasses, t->HeadParts[HEAD_PART_GLASSES]);
 		}
 		pd.Colors = CharColors2Net(t->Colors);
 	}
@@ -171,41 +194,42 @@ NPlayerData PlayerDataDefault(const int idx)
 			pd.Colors.Arms = Color2Net(colorLightBlue);
 			pd.Colors.Body = Color2Net(colorLightBlue);
 			pd.Colors.Legs = Color2Net(colorLightBlue);
-			pd.Colors.Hair = Color2Net(colorLightBlue);
 			pd.Colors.Feet = Color2Net(colorLightBlue);
 			break;
 		case 1:
 			strcpy(pd.Name, "Ice");
 			strcpy(pd.CharacterClass, "Jones");
-			strcpy(pd.Hair, "shades");
+			strcpy(pd.Glasses, "shades");
 			pd.Colors.Skin = Color2Net(colorDarkSkin);
 			pd.Colors.Arms = Color2Net(colorRed);
 			pd.Colors.Body = Color2Net(colorRed);
 			pd.Colors.Legs = Color2Net(colorRed);
-			pd.Colors.Hair = Color2Net(colorBlack);
 			pd.Colors.Feet = Color2Net(colorRed);
+			pd.Colors.Glasses = Color2Net(colorBlack);
 			break;
 		case 2:
 			strcpy(pd.Name, "Warbaby");
 			strcpy(pd.CharacterClass, "Jones");
-			strcpy(pd.Hair, "beret");
+			strcpy(pd.Hat, "beret");
 			pd.Colors.Skin = Color2Net(colorSkin);
 			pd.Colors.Arms = Color2Net(colorGreen);
 			pd.Colors.Body = Color2Net(colorGreen);
 			pd.Colors.Legs = Color2Net(colorGreen);
-			pd.Colors.Hair = Color2Net(colorRed);
 			pd.Colors.Feet = Color2Net(colorGreen);
+			pd.Colors.Hat = Color2Net(colorRed);
 			break;
 		case 3:
 			strcpy(pd.Name, "Han");
 			strcpy(pd.CharacterClass, "Jones");
-			strcpy(pd.Hair, "hogan");
+			strcpy(pd.Hair, "rattail");
+			strcpy(pd.Facehair, "handlebar");
 			pd.Colors.Skin = Color2Net(colorAsianSkin);
 			pd.Colors.Arms = Color2Net(colorYellow);
 			pd.Colors.Body = Color2Net(colorYellow);
 			pd.Colors.Legs = Color2Net(colorYellow);
 			pd.Colors.Hair = Color2Net(colorYellow);
 			pd.Colors.Feet = Color2Net(colorYellow);
+			pd.Colors.Facehair = Color2Net(colorYellow);
 			break;
 		default:
 			// Set up player N template
@@ -221,10 +245,12 @@ NPlayerData PlayerDataDefault(const int idx)
 		}
 	}
 
-	pd.MaxHealth = 200;
+	pd.HP = CampaignGetHP(&gCampaign);
+	pd.MaxHealth = CampaignGetMaxHP(&gCampaign);
+	pd.ExcessHealth = CampaignGetExcessHP(&gCampaign);
+	pd.Lives = CampaignGetLives(&gCampaign);
 	if (gCampaign.Setting.BuyAndSell)
 	{
-		pd.Stats.Score = STARTING_CASH;
 		pd.Totals.Score = STARTING_CASH;
 	}
 
@@ -236,12 +262,16 @@ NPlayerData PlayerDataDefault(const int idx)
 NPlayerData PlayerDataMissionReset(const PlayerData *p)
 {
 	NPlayerData pd = NMakePlayerData(p);
-	pd.Lives = ModeLives(gCampaign.Entry.Mode);
+	if (gCampaign.Setting.PlayerHP == 0)
+	{
+		pd.HP = CampaignGetHP(&gCampaign);
+	}
 
 	memset(&pd.Stats, 0, sizeof pd.Stats);
 
 	pd.LastMission = gCampaign.MissionIndex;
-	pd.MaxHealth = ModeMaxHealth(gCampaign.Entry.Mode);
+	pd.MaxHealth = CampaignGetMaxHP(&gCampaign);
+	pd.ExcessHealth = CampaignGetExcessHP(&gCampaign);
 	return pd;
 }
 
@@ -495,11 +525,15 @@ int PlayerGetNumWeapons(const PlayerData *p)
 
 bool PlayerHasGrenadeButton(const PlayerData *p)
 {
-	return InputHasGrenadeButton(p->inputDevice, p->deviceIndex);
+	return p && InputHasGrenadeButton(p->inputDevice, p->deviceIndex);
 }
 
 bool PlayerHasWeapon(const PlayerData *p, const WeaponClass *wc)
 {
+	if (wc == NULL)
+	{
+		return false;
+	}
 	for (int i = 0; i < MAX_WEAPONS; i++)
 	{
 		if (p->guns[i] == wc)
@@ -510,6 +544,23 @@ bool PlayerHasWeapon(const PlayerData *p, const WeaponClass *wc)
 	return false;
 }
 
+bool PlayerHasWeaponUpgrade(const PlayerData *p, const WeaponClass *wc)
+{
+	if (wc == NULL)
+	{
+		return false;
+	}
+	for (int i = 0; i < MAX_WEAPONS; i++)
+	{
+		const WeaponClass *prereq = WeaponClassGetPrerequisite(p->guns[i]);
+		if (prereq == wc)
+		{
+			return true;
+		}
+	}
+	return false;
+}
+
 static void AddWeaponToSlot(
 	PlayerData *p, const WeaponClass *wc, const int slot)
 {
@@ -537,13 +588,20 @@ void PlayerAddWeaponToSlot(
 {
 	if (wc)
 	{
-		// See if the weapon is already equipped; if so swap it with the slot
+		// See if the weapon or its pre-/post-requisite is already
+		// equipped; if so swap it with the slot
 		for (int i = 0; i < MAX_WEAPONS; i++)
 		{
-			if (p->guns[i] == wc)
+			if (i == slot)
+			{
+				continue;
+			}
+			if (p->guns[i] && (p->guns[i] == wc ||
+							   WeaponClassGetPrerequisite(p->guns[i]) == wc ||
+							   p->guns[i] == WeaponClassGetPrerequisite(wc)))
 			{
 				p->guns[i] = p->guns[slot];
-				p->guns[slot] = wc;
+				PlayerAddWeaponToSlot(p, wc, i);
 				return;
 			}
 		}
@@ -554,9 +612,9 @@ void PlayerAddWeaponToSlot(
 		PlayerRemoveWeapon(p, slot);
 	}
 	// Subtract gun price
-	if (gCampaign.Setting.BuyAndSell && wc)
+	if (gCampaign.Setting.BuyAndSell)
 	{
-		PlayerScore(p, -wc->Price);
+		PlayerScore(p, -WeaponClassFullPrice(wc));
 	}
 	AddWeaponToSlot(p, wc, slot);
 }
@@ -593,9 +651,9 @@ void PlayerAddWeapon(PlayerData *p, const WeaponClass *wc)
 void PlayerRemoveWeapon(PlayerData *p, const int slot)
 {
 	// Refund gun price
-	if (gCampaign.Setting.BuyAndSell && p->guns[slot])
+	if (gCampaign.Setting.BuyAndSell)
 	{
-		PlayerScore(p, p->guns[slot]->Price);
+		PlayerScore(p, WeaponClassFullPrice(p->guns[slot]));
 	}
 	// Refund ammo if no guns use this ammo
 	int ammoIds[MAX_BARRELS] = {-1, -1};
@@ -684,3 +742,27 @@ void PlayerAddAmmo(
 		PlayerScore(p, dLots * a->Price);
 	}
 }
+
+void PlayerSetHP(PlayerData *p, const int hp)
+{
+	p->HP = CLAMP(hp, 1, CampaignGetMaxHP(&gCampaign));
+}
+
+void PlayerSetLives(PlayerData *p, const int lives)
+{
+	p->Lives = CLAMP(lives, 1, CampaignGetMaxLives(&gCampaign));
+}
+
+int CharacterGetStartingHealth(const Character *c, const PlayerData *p)
+{
+	if (p == NULL)
+	{
+		return MAX(
+			(c->maxHealth * ConfigGetInt(&gConfig, "Game.NonPlayerHP")) / 100,
+			1);
+	}
+	else
+	{
+		return p->HP > 0 ? p->HP : c->maxHealth;
+	}
+}
